@@ -1,4 +1,4 @@
-/* Copyright 2007 - 2009 Matthew Barnes
+/* Copyright 2007-2011 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -24,10 +24,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef HAVE_GNOME
-#include <gnome.h>
-#endif
-
 #include "gva-audit.h"
 #include "gva-categories.h"
 #include "gva-db.h"
@@ -43,13 +39,12 @@
 #include "gva-ui.h"
 #include "gva-util.h"
 
-#ifdef HAVE_DBUS
-#include "gva-dbus.h"
-#endif
+#define APPLICATION_ID "org.gnome.VideoArcade"
 
 #define SQL_COUNT_ROMS \
         "SELECT count(*) FROM game WHERE " \
-        "romset NOTNULL AND romset != 'not found'"
+        "romset NOTNULL AND romset != 'not found' " \
+        "AND isbios != 'yes'"
 
 static GOptionEntry entries[] =
 {
@@ -109,7 +104,6 @@ warn_if_no_roms (void)
                 "<big><b>%s</b></big>",
                 _("No ROM files found"));
 
-#if GTK_CHECK_VERSION(2,14,0) || defined HAVE_GNOME
         gtk_message_dialog_format_secondary_markup (
                 GTK_MESSAGE_DIALOG (dialog),
                 _("GNOME Video Arcade was unable to locate any ROM files. "
@@ -119,14 +113,6 @@ warn_if_no_roms (void)
 
         gtk_dialog_add_button (
                 GTK_DIALOG (dialog), GTK_STOCK_HELP, GTK_RESPONSE_HELP);
-#else
-        gtk_message_dialog_format_secondary_markup (
-                GTK_MESSAGE_DIALOG (dialog),
-                _("GNOME Video Arcade was unable to locate any ROM files. "
-                  "It could be that MAME is misconfigured or no ROM files "
-                  "are installed. Consult the user documentation for more "
-                  "details and troubleshooting tips."));
-#endif
 
         if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_HELP)
                 gva_help_display (GTK_WINDOW (dialog), "troubleshooting");
@@ -134,79 +120,167 @@ warn_if_no_roms (void)
         gtk_widget_destroy (dialog);
 }
 
-static void
+static gboolean
 start (void)
 {
+        GSettings *settings;
         GError *error = NULL;
 
+        settings = gva_get_settings ();
+
         if (gva_db_needs_rebuilt ())
         {
-                gva_main_progress_bar_show ();
-
                 if (!gva_main_build_database (&error))
-                {
-                        gva_error_handle (&error);
-                        return;
-                }
-
-                gva_main_progress_bar_set_fraction (0.0);
+                        goto exit;
 
                 if (!gva_main_analyze_roms (&error))
-                {
-                        gva_error_handle (&error);
-                        return;
-                }
-
-                gva_main_progress_bar_hide ();
+                        goto exit;
         }
         else if (gva_audit_detect_changes ())
         {
-                gva_main_progress_bar_show ();
-
                 if (!gva_main_analyze_roms (&error))
-                {
-                        gva_error_handle (&error);
-                        return;
-                }
-
-                gva_main_progress_bar_hide ();
+                        goto exit;
         }
 
         /* Do this after ROMs are analyzed. */
         if (!gva_main_init_search_completion (&error))
-        {
-                gva_error_handle (&error);
-                return;
-        }
+                goto exit;
 
-        gtk_action_set_sensitive (GVA_ACTION_SEARCH, TRUE);
-        gtk_action_set_sensitive (GVA_ACTION_SHOW_CLONES, TRUE);
-        gtk_action_set_sensitive (GVA_ACTION_VIEW_AVAILABLE, TRUE);
-        gtk_action_set_sensitive (GVA_ACTION_VIEW_FAVORITES, TRUE);
-        gtk_action_set_sensitive (GVA_ACTION_VIEW_RESULTS, TRUE);
+        gva_ui_unlock ();
 
-        gconf_bridge_bind_property (
-                gconf_bridge_get (), GVA_GCONF_SELECTED_VIEW_KEY,
-                G_OBJECT (GVA_ACTION_VIEW_AVAILABLE), "current-value");
-
-        /* Force a tree view update. */
-        if (gva_tree_view_get_selected_view () == 0)
-        {
-                gva_tree_view_update (&error);
-                gva_error_handle (&error);
-        }
+        g_settings_bind (
+                settings, GVA_SETTING_SELECTED_VIEW,
+                GVA_ACTION_VIEW_AVAILABLE, "current-value",
+                G_SETTINGS_BIND_DEFAULT);
 
         /* Present a helpful dialog if no ROMs were found. */
         warn_if_no_roms ();
+
+exit:
+        gva_error_handle (&error);
+
+        return FALSE;
+}
+
+static void
+rompath_changed_cb (GFileMonitor *monitor,
+                    GFile *file,
+                    GFile *other_file,
+                    GFileMonitorEvent event_type)
+{
+        static GtkWidget *dialog = NULL;
+        gint response;
+
+        if (dialog != NULL)
+                return;
+
+        /* Filter out events we don't care about. */
+        switch (event_type)
+        {
+                case G_FILE_MONITOR_EVENT_CHANGED:
+                case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
+                case G_FILE_MONITOR_EVENT_DELETED:
+                case G_FILE_MONITOR_EVENT_CREATED:
+                        break;
+                default:
+                        return;
+        }
+
+        dialog = gtk_message_dialog_new_with_markup (
+                GTK_WINDOW (GVA_WIDGET_MAIN_WINDOW),
+                GTK_DIALOG_DESTROY_WITH_PARENT,
+                GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
+                "<big><b>%s</b></big>",
+                _("Changes detected in ROM files"));
+
+        gtk_message_dialog_format_secondary_text (
+                GTK_MESSAGE_DIALOG (dialog),
+                _("GNOME Video Arcade has detected changes in your "
+                  "ROM files and will need to audit them in order to "
+                  "update the game list. However the audit may take "
+                  "several minutes to complete. Would you like to "
+                  "perform the audit now?\n\nIf you skip the audit now, "
+                  "it will be performed automatically the next time you "
+                  "start GNOME Video Arcade."));
+
+        gtk_dialog_add_buttons (
+                GTK_DIALOG (dialog),
+                _("_Skip Audit"), GTK_RESPONSE_NO,
+                _("_Audit ROM Files"), GTK_RESPONSE_YES,
+                NULL);
+
+        response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+        /* Don't destroy the dialog just yet.  If the file monitor
+         * trips again while we're analyzing ROMs, we want the NULL
+         * check at the top of the function to evaluate TRUE so we
+         * return immediately. */
+        gtk_widget_hide (dialog);
+
+        if (response == GTK_RESPONSE_YES)
+        {
+                GError *error = NULL;
+
+                gva_ui_lock ();
+
+                gva_main_analyze_roms (&error);
+                gva_error_handle (&error);
+
+                gva_tree_view_update (&error);
+                gva_error_handle (&error);
+
+                gva_ui_unlock ();
+
+                /* Present a helpful dialog if no ROMs were found. */
+                warn_if_no_roms ();
+        }
+
+        gtk_widget_destroy (dialog);
+        dialog = NULL;
+}
+
+static gboolean
+setup_file_monitors (void)
+{
+        gchar **search_paths;
+        guint length, ii;
+
+        /* We don't care about errors while setting up file monitors
+         * since it's just a "nice to have" feature. */
+
+        search_paths = gva_mame_get_search_paths ("rompath", NULL);
+        length = (search_paths != NULL) ? g_strv_length (search_paths) : 0;
+
+        for (ii = 0; ii < length; ii++)
+        {
+                GFileMonitor *monitor;
+                GFile *file;
+
+                file = g_file_new_for_path (search_paths[ii]);
+                monitor = g_file_monitor (file, 0, NULL, NULL);
+                g_object_unref (file);
+
+                if (monitor == NULL)
+                        continue;
+
+                g_signal_connect (
+                        monitor, "changed",
+                        G_CALLBACK (rompath_changed_cb), NULL);
+
+                /* We leak the file monitor reference, but the
+                 * monitor lives for the entire session anyway. */
+        }
+
+        g_strfreev (search_paths);
+
+        return FALSE;
 }
 
 gint
 main (gint argc, gchar **argv)
 {
-#ifdef HAVE_GNOME
-        GnomeProgram *program;
-        GOptionContext *context;
-#endif
+        GtkApplication *application;
+        GApplicationFlags flags;
         gchar *path;
         GError *error = NULL;
 
@@ -216,23 +290,19 @@ main (gint argc, gchar **argv)
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
         textdomain (GETTEXT_PACKAGE);
 
-#ifdef HAVE_GNOME
-        context = g_option_context_new (NULL);
-        g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
-        g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
-
-        program = gnome_program_init (
-                PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
-                GNOME_PROGRAM_STANDARD_PROPERTIES,
-                GNOME_PARAM_GOPTION_CONTEXT, context,
-                GNOME_PARAM_NONE);
-#else
         gtk_init_with_args (
                 &argc, &argv, NULL, entries, GETTEXT_PACKAGE, &error);
         if (error != NULL)
                 g_error ("%s", error->message);
+
+#ifdef HAVE_GSTREAMER
+        if (!gst_init_check (&argc, &argv, &error))
+                g_error ("%s", error->message);
 #endif
 
+        /* This installs handlers for our custom debug log levels. */
+        gva_get_debug_flags ();
+
         /* Change the working directory to that of the MAME executable.
          * Why?  Because SDLMAME's default configuration uses relative
          * search paths such as "rompath = roms".  The paths are relative
@@ -258,19 +328,57 @@ main (gint argc, gchar **argv)
                         g_printerr ("%s\n", error->message);
                         g_clear_error (&error);
                 }
-                exit (0);
+                exit (EXIT_SUCCESS);
         }
 
         if (opt_version)
         {
                 g_print ("%s\n", PACKAGE_STRING);
-                exit (0);
+                exit (EXIT_SUCCESS);
         }
 
         if (opt_which_emulator)
         {
                 g_print ("%s\n", MAME_PROGRAM);
-                exit (0);
+                exit (EXIT_SUCCESS);
+        }
+
+        /* Register the application with the session bus. */
+        flags = G_APPLICATION_FLAGS_NONE;
+        application = gtk_application_new (APPLICATION_ID, flags);
+        g_application_register (G_APPLICATION (application), NULL, &error);
+
+        if (error != NULL)
+                g_error ("%s", error->message);
+
+        /* If another instance is running, exit now. */
+        if (g_application_get_is_remote (G_APPLICATION (application)))
+        {
+                gint exit_status;
+
+                if (opt_build_database)
+                {
+                        g_printerr (
+                                "Cannot build database: "
+                                PACKAGE_NAME " is already running\n");
+                        exit_status = EXIT_FAILURE;
+                }
+                else
+                {
+                        g_application_activate (G_APPLICATION (application));
+                        exit_status = EXIT_SUCCESS;
+                }
+
+                g_object_unref (application);
+
+                exit (exit_status);
+        }
+        else
+        {
+                GtkWindow *window;
+
+                window = GTK_WINDOW (GVA_WIDGET_MAIN_WINDOW);
+                gtk_application_add_window (application, window);
         }
 
         gtk_window_set_default_icon_name (PACKAGE);
@@ -283,6 +391,7 @@ main (gint argc, gchar **argv)
         gva_play_back_init ();
         gva_preferences_init ();
         gva_properties_init ();
+        gva_ui_init ();
 
         gva_categories_init (&error);
         gva_error_handle (&error);
@@ -293,18 +402,12 @@ main (gint argc, gchar **argv)
         gva_nplayers_init (&error);
         gva_error_handle (&error);
 
-#ifdef HAVE_DBUS
-        gva_dbus_init (&error);
-        gva_error_handle (&error);
-#endif
-
-        gtk_init_add ((GtkFunction) start, NULL);
+        g_idle_add ((GSourceFunc) start, NULL);
+        g_idle_add ((GSourceFunc) setup_file_monitors, NULL);
 
         gtk_main ();
 
-#ifdef HAVE_GNOME
-        g_object_unref (program);
-#endif
+        g_object_unref (application);
 
         return EXIT_SUCCESS;
 }
